"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const lodash_1 = require("lodash");
const updateQueueBuffer_1 = __importDefault(require("../functions/updateQueueBuffer"));
const socketDataEncoder_1 = __importDefault(require("../functions/socketDataEncoder"));
const isImarketCodes_1 = __importDefault(require("../functions/isImarketCodes"));
/**
 * useWsTrade is a custom hook that connects to a WebSocket API
 * and retrieves real-time trade data for a given market code.
 * @param targetMarketCodes - Array of market codes to retrieve trade data for.
 * @param options - `throttle_time` the data update frequency(ms).
 * @param options - `max_length_queue` Maximum number of items in data buffer.
 * @throws targetMarketCodes should be React State Value, if not, unexpected errors can occur.
 * @returns Object with the WebSocket object, connection status, and real-time trade data.
 */
function useWsTrade(targetMarketCodes, onError, options = {}) {
    const { throttle_time = 400, max_length_queue = 100, debug = false } = options;
    const SOCKET_URL = 'wss://api.upbit.com/websocket/v1';
    const socket = (0, react_1.useRef)(null);
    const buffer = (0, react_1.useRef)([]);
    const [isConnected, setIsConnected] = (0, react_1.useState)(false);
    const [socketData, setSocketData] = (0, react_1.useState)();
    const throttled = (0, lodash_1.throttle)(() => {
        try {
            const updatedBuffer = (0, updateQueueBuffer_1.default)(buffer.current, max_length_queue);
            buffer.current = updatedBuffer;
            setSocketData(updatedBuffer);
        }
        catch (error) {
            console.error(error);
        }
    }, throttle_time);
    // socket 세팅
    (0, react_1.useEffect)(() => {
        try {
            if (targetMarketCodes && !(0, isImarketCodes_1.default)(targetMarketCodes)) {
                throw new Error('targetMarketCodes does not have the correct interface');
            }
            if ([targetMarketCodes].length > 0 && !socket.current) {
                socket.current = new WebSocket(SOCKET_URL);
                socket.current.binaryType = 'arraybuffer';
                const socketOpenHandler = () => {
                    var _a;
                    setIsConnected(true);
                    if (debug)
                        console.log('[completed connect] | socket Open Type: ', 'trade');
                    if (((_a = socket.current) === null || _a === void 0 ? void 0 : _a.readyState) == 1) {
                        const sendContent = [
                            { ticket: 'test' },
                            {
                                type: 'trade',
                                codes: [targetMarketCodes.market],
                            },
                        ];
                        socket.current.send(JSON.stringify(sendContent));
                        if (debug)
                            console.log('message sending done');
                    }
                };
                const socketCloseHandler = () => {
                    setIsConnected(false);
                    setSocketData([]);
                    buffer.current = [];
                    if (debug)
                        console.log('connection closed');
                };
                const socketErrorHandler = (event) => {
                    const error = event.error;
                    console.error('[Error]', error);
                };
                const socketMessageHandler = (evt) => {
                    const data = (0, socketDataEncoder_1.default)(evt.data);
                    data && buffer.current.push(data);
                    throttled();
                };
                socket.current.onopen = socketOpenHandler;
                socket.current.onclose = socketCloseHandler;
                socket.current.onerror = socketErrorHandler;
                socket.current.onmessage = socketMessageHandler;
            }
            return () => {
                if (socket.current) {
                    if (socket.current.readyState != 0) {
                        socket.current.close();
                        socket.current = null;
                    }
                }
            };
        }
        catch (error) {
            if (error instanceof Error) {
                if (onError) {
                    onError(error);
                }
                else {
                    console.error(error);
                    throw error;
                }
            }
        }
    }, [targetMarketCodes]);
    return { socket: socket.current, isConnected, socketData };
}
exports.default = useWsTrade;
